import json
import requests
from functools import reduce


def __reduce(ok_licences, notok_licenses):
    def inner_reduce(res, x):
        if x.lower() in ok_licences:
            res['ok'].append(x)
        elif x.lower() in notok_licenses:
            res['notok'].append(x)
        else:
            res['unknown'].append(x)
        return res
    return inner_reduce


def are_valid(license_file):
    with open(license_file, 'r') as f:
        licenses = json.loads(f.read())

    license_reference = requests.get('https://licenses.miquido.dev').json()
    ok_licenses = set(map(lambda x: x.lower(), license_reference['ok']))
    notok_licenses = set(map(lambda x: x.lower(), license_reference['notok']))
    result = {
        'ok': [],
        'notok': [],
        'unknown': []
    }

    licenses_names = list(map(lambda x: x['id'], licenses['licenses']))

    return reduce(__reduce(ok_licenses, notok_licenses), licenses_names, result)



import json
import unittest
from unittest import mock
from miquido_license_validator import license_validator


class MockResponse:
    def __init__(self, status_code, json_body: dict):
        self.json_body = json_body
        self.status_code = status_code

    def json(self):
        return self.json_body


def mocked_request(*args, **kwargs):
    with open('test/fixtures/licenses_reference.json', 'r') as file:
        res = file.read()
    return MockResponse(200, json.loads(res))


class LicenseVerifierTests(unittest.TestCase):

    @mock.patch('requests.get', side_effect=mocked_request)
    def test_license_verifier(self, mock):
        res = license_validator.are_valid('test/fixtures/gl-license-scanning-report.json')
        self.assertEqual(1, len(res['ok']))
        self.assertEqual(1, len(res['notok']))
        self.assertEqual(6, len(res['unknown']))
        self.assertEqual("MIT", res['ok'][0])
        self.assertEqual("bsd", res['notok'][0])
